# coding: utf-8
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.


"""
    knext

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from knext.common.rest.configuration import Configuration


class OperatorConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "file_path": "str",
        "module_path": "int",
        "class_name": "str",
        "method": "str",
        "params": "object",
    }

    attribute_map = {
        "file_path": "filePath",
        "module_path": "modulePath",
        "class_name": "className",
        "method": "method",
        "params": "params",
    }

    def __init__(
        self,
        file_path=None,
        module_path=None,
        class_name=None,
        method=None,
        params=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """OperatorConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._file_path = None
        self._module_path = None
        self._class_name = None
        self._method = None
        self._params = None
        self.discriminator = None

        if file_path is not None:
            self.file_path = file_path
        if module_path is not None:
            self.module_path = module_path
        if class_name is not None:
            self.class_name = class_name
        if method is not None:
            self.method = method
        if params is not None:
            self.params = params

    @property
    def file_path(self):
        """Gets the file_path of this OperatorConfig.  # noqa: E501


        :return: The file_path of this OperatorConfig.  # noqa: E501
        :rtype: str
        """
        return self._file_path

    @file_path.setter
    def file_path(self, file_path):
        """Sets the file_path of this OperatorConfig.


        :param file_path: The file_path of this OperatorConfig.  # noqa: E501
        :type: str
        """

        self._file_path = file_path

    @property
    def module_path(self):
        """Gets the module_path of this OperatorConfig.  # noqa: E501


        :return: The module_path of this OperatorConfig.  # noqa: E501
        :rtype: int
        """
        return self._module_path

    @module_path.setter
    def module_path(self, module_path):
        """Sets the module_path of this OperatorConfig.


        :param module_path: The module_path of this OperatorConfig.  # noqa: E501
        :type: int
        """

        self._module_path = module_path

    @property
    def class_name(self):
        """Gets the class_name of this OperatorConfig.  # noqa: E501


        :return: The class_name of this OperatorConfig.  # noqa: E501
        :rtype: str
        """
        return self._class_name

    @class_name.setter
    def class_name(self, class_name):
        """Sets the class_name of this OperatorConfig.


        :param class_name: The class_name of this OperatorConfig.  # noqa: E501
        :type: str
        """

        self._class_name = class_name

    @property
    def method(self):
        """Gets the method of this OperatorConfig.  # noqa: E501


        :return: The method of this OperatorConfig.  # noqa: E501
        :rtype: str
        """
        return self._method

    @method.setter
    def method(self, method):
        """Sets the method of this OperatorConfig.


        :param method: The method of this OperatorConfig.  # noqa: E501
        :type: str
        """

        self._method = method

    @property
    def params(self):
        """Gets the params of this OperatorConfig.  # noqa: E501


        :return: The params of this OperatorConfig.  # noqa: E501
        :rtype: object
        """
        return self._params

    @params.setter
    def params(self, params):
        """Sets the params of this OperatorConfig.


        :param params: The params of this OperatorConfig.  # noqa: E501
        :type: object
        """

        self._params = params

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OperatorConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OperatorConfig):
            return True

        return self.to_dict() != other.to_dict()
