# coding: utf-8
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.


"""
    knext

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from knext.common.rest.configuration import Configuration


class BuilderJobSubmitRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "job_name": "str",
        "project_id": "int",
        "pipeline": "Pipeline",
        "cron": "str",
        "idempotent_id": "str",
        "params": "object",
    }

    attribute_map = {
        "job_name": "jobName",
        "project_id": "projectId",
        "pipeline": "pipeline",
        "cron": "cron",
        "idempotent_id": "idempotentId",
        "params": "params",
    }

    def __init__(
        self,
        job_name=None,
        project_id=None,
        pipeline=None,
        cron=None,
        idempotent_id=None,
        params=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """BuilderJobSubmitRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._job_name = None
        self._project_id = None
        self._pipeline = None
        self._cron = None
        self._idempotent_id = None
        self._params = None
        self.discriminator = None

        if job_name is not None:
            self.job_name = job_name
        if project_id is not None:
            self.project_id = project_id
        if pipeline is not None:
            self.pipeline = pipeline
        if cron is not None:
            self.cron = cron
        if idempotent_id is not None:
            self.idempotent_id = idempotent_id
        if params is not None:
            self.params = params

    @property
    def job_name(self):
        """Gets the job_name of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The job_name of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: str
        """
        return self._job_name

    @job_name.setter
    def job_name(self, job_name):
        """Sets the job_name of this BuilderJobSubmitRequest.


        :param job_name: The job_name of this BuilderJobSubmitRequest.  # noqa: E501
        :type: str
        """

        self._job_name = job_name

    @property
    def project_id(self):
        """Gets the project_id of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The project_id of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: int
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this BuilderJobSubmitRequest.


        :param project_id: The project_id of this BuilderJobSubmitRequest.  # noqa: E501
        :type: int
        """

        self._project_id = project_id

    @property
    def pipeline(self):
        """Gets the pipeline of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The pipeline of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: Pipeline
        """
        return self._pipeline

    @pipeline.setter
    def pipeline(self, pipeline):
        """Sets the pipeline of this BuilderJobSubmitRequest.


        :param pipeline: The pipeline of this BuilderJobSubmitRequest.  # noqa: E501
        :type: Pipeline
        """

        self._pipeline = pipeline

    @property
    def cron(self):
        """Gets the cron of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The cron of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: str
        """
        return self._cron

    @cron.setter
    def cron(self, cron):
        """Sets the cron of this BuilderJobSubmitRequest.


        :param cron: The cron of this BuilderJobSubmitRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["SYNC", "ONCE", "CRON"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and cron not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `cron` ({0}), must be one of {1}".format(  # noqa: E501
                    cron, allowed_values
                )
            )

        self._cron = cron

    @property
    def idempotent_id(self):
        """Gets the idempotent_id of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The idempotent_id of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: str
        """
        return self._idempotent_id

    @idempotent_id.setter
    def idempotent_id(self, idempotent_id):
        """Sets the idempotent_id of this BuilderJobSubmitRequest.


        :param idempotent_id: The idempotent_id of this BuilderJobSubmitRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["LOCAL", "REMOTE"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and idempotent_id not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `idempotent_id` ({0}), must be one of {1}".format(  # noqa: E501
                    idempotent_id, allowed_values
                )
            )

        self._idempotent_id = idempotent_id

    @property
    def params(self):
        """Gets the params of this BuilderJobSubmitRequest.  # noqa: E501


        :return: The params of this BuilderJobSubmitRequest.  # noqa: E501
        :rtype: object
        """
        return self._params

    @params.setter
    def params(self, params):
        """Sets the params of this BuilderJobSubmitRequest.


        :param params: The params of this BuilderJobSubmitRequest.  # noqa: E501
        :type: object
        """

        self._params = params

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BuilderJobSubmitRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BuilderJobSubmitRequest):
            return True

        return self.to_dict() != other.to_dict()
